# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-05-17 08:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0046_remove_unique_constraints'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='hide_course_original_price',
            field=models.BooleanField(default=False, help_text='Specify whether display the course original price on enterprise course landing page or not.'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='hide_course_original_price',
            field=models.BooleanField(default=False, help_text='Specify whether display the course original price on enterprise course landing page or not.'),
        ),
    ]
